import java.util.*;

/*
  This example loads in all the transactions (that are located
  in .txt files within the data directory) that have been 
  created by the xmlStripper, assuming that the xmlStripper
  did not filter out any fields. In this example we load in all the fields
  for each transaction, and draw the title to the screen. We can load
  about 1,000,000 transactions in 10 seconds. Loading in this
  much data may require increasing the maximum available memory.
*/

ArrayList transactions = null; //global List of the data
int idx = 0;
int maxNum = 200;

void setup()
{
  size(600,400);
  frame.setResizable(true);
  background(0);
  frameRate(1000); //that is, as fast as possible

  textFont(loadFont("GillSans-64.vlw"), 64); 
  long before = millis();
  this.transactions = loadTransactions();

  println("loaded " + this.transactions.size() + " transactions in " + (millis() - before) + " miiliseconds.");
 
  TransactionUtils.sortTransactionsByTitle(transactions);
  TransactionUtils.reverse(transactions);
 
  for (int i = 0; i < 50; i++)
  {
    println(transactions.get(i)); 
  }  
}

void draw()
{
  Transaction t = (Transaction) transactions.get(idx);

  textAlign(CENTER);
  fill(random(0,255),random(0,255),random(0,255),random(100,255));
  text(t.title, random(0, width), random(0, height));

  if (idx < maxNum)
  {
    delay(maxNum - (idx));
  }

  idx = (idx+1)%transactions.size();
  if (idx == 0)
  {
    background(0);
  }

  println(frameRate);
}

  

public ArrayList loadTransactions()
{
  ArrayList transactions = new ArrayList();

  File[] fileArr = listFiles(dataPath(""));

  for (int fn = 0; fn < fileArr.length; fn++) 
  {
    File f = (File) fileArr[fn];    

    if (!f.getName().endsWith(".txt"))
    {
      continue;
    }

    println("loading file... \"" + f.getName() + "\"");
    String[] lines = loadStrings(f);

    for (int i = 0; i < lines.length; i++)
    {
      transactions.add(parseTransaction(lines[i])); 
    }
  }

  return transactions;
}


// This function returns all the files in a directory as an array of File objects
// This is useful if you want more info about the file
public File[] listFiles(String dir) {
  File file = new File(dir);
  if (file.isDirectory()) {
    File[] files = file.listFiles();
    return files;
  } 
  else {
    // If it's not a directory
    return null;
  }
}

  //create a new transaction obejct from a comma separated line.
public Transaction parseTransaction(String line)
{
  String sections[] = split(line, ",");

  Transaction t = new Transaction();

  //store all fields
  t.itemNumber = sections[0];
  t.bibNumber = sections[1];
  t.ckodate = sections[2];
  t.ckotime = sections[3];
  t.ckidate = sections[4];
  t.ckitime = sections[5];
  t.collcode = sections[6];
  t.itemtype = sections[7];
  t.barcode = sections[8];
  t.title = sections[9];
  t.callNumber = sections[10];
  t.deweyClass = sections[11];
 
  int numSubjects = sections.length - 12;
  t.subjects = new String[numSubjects];
  arraycopy(sections, 12, t.subjects, 0, numSubjects);

  return t;
}






